@@ -22,6 +22,7 @@ from datetime import datetime
 from random import choice
 from xml.etree.ElementTree import Element, SubElement, tostring
 
+
 # Cuidado de respetar el orden !!!!!!!!!
 # En caso de cambiar el orden podria generar
 # problemas al validar comprobantes en Uruware
@@ -61,6 +62,30 @@ MESSAGES_TYPES = {
 
 LOCAL_COUNTRY_CODE = 'UY'
 
+def round_half_away_from_zero(n, digits=0):
+    """Devuelve n redondeado a digits dígitos decimales,
+       minimizar los errores de representación de punto flotante IEEE-754 y aplicar
+       HALF-UP (away from zero).
+       Créditos: Función replicada de float_round de odoo/tools/float_utils 
+       en repositorio odoo rama 13.0 aplicando solo HALF-UP (away from zero)
+
+    :param n: valor a redondear
+    :type n: float
+    :param digits: número de dígitos decimales a redondear, defaults to 0
+    :type digits: int, optional
+    :return: valor redondeado
+    :rtype: flaot
+    """
+    if digits == 0 or n == 0:
+        return 0.0
+    rounding_factor = 10 ** -digits
+    normalized_value = n / rounding_factor
+    epsilon_magnitude = math.log(abs(normalized_value), 2)
+    epsilon = 2**(epsilon_magnitude-52)
+    normalized_value += math.copysign(epsilon, normalized_value)
+    rounded_value = round(normalized_value)
+    return rounded_value * rounding_factor
+
 
 class Manager(object):
     __slots__ = ['emisor', 'comprobante', 'connection_data']
@@ -511,12 +536,12 @@ class Manager(object):
 
     @staticmethod
     def _format_tipo_cambio(rate):
-        return '{:.3f}'.format(rate)
+        return '{:.3f}'.format(round_half_away_from_zero(rate, 3))
 
     @staticmethod
-    def _format_amount(rate, decimal=2):
+    def _format_amount(amount, decimal=2):
         value = '{:.' + str(decimal) + 'f}'
-        return value.format(rate)
+        return value.format(round_half_away_from_zero(amount, 2))
 
     def _get_uuid_cfe(self):
         uuid = self.comprobante.uuid if hasattr(self.comprobante, 'uuid') else False
@@ -576,15 +601,15 @@ class Manager(object):
         return self.get_tax(INVOICE_INDICATOR.get('taxable_basic'))
 
     def get_total_amount(self):
-        total_basic = round(self.get_mnt_iva_tasa_basic(),2) + round(self.get_mnt_base_iva_tasa_basic(),2)
-        total_min = round(self.get_mnt_iva_tasa_min(),2) + round(self.get_mnt_base_iva_tasa_min(),2)
-        total_no_taxable = round(self.get_mnt_non_taxable(),2)
-        total_exempt = round(self.get_mnt_exempt(),2)
-        total_exportation = round(self.get_mnt_exportation(),2)
+        total_basic = round_half_away_from_zero(self.get_mnt_iva_tasa_basic(), 2) + round_half_away_from_zero(self.get_mnt_base_iva_tasa_basic(), 2)
+        total_min = round_half_away_from_zero(self.get_mnt_iva_tasa_min(), 2) + round_half_away_from_zero(self.get_mnt_base_iva_tasa_min(), 2)
+        total_no_taxable = round_half_away_from_zero(self.get_mnt_non_taxable(), 2)
+        total_exempt = round_half_away_from_zero(self.get_mnt_exempt(), 2)
+        total_exportation = round_half_away_from_zero(self.get_mnt_exportation(), 2)
         return total_basic + total_min + total_no_taxable + total_exempt + total_exportation
 
     def get_total_to_pay(self):
-        return self.get_total_amount() + round(self.get_mnt_non_invoiceable(),2) + round(self.get_mnt_negative_non_invoiceable(),2)
+        return self.get_total_amount() + round_half_away_from_zero(self.get_mnt_non_invoiceable(),2) + round_half_away_from_zero(self.get_mnt_negative_non_invoiceable(),2)
 
     def get_total_retention(self):
         return sum([ret.get_amount() for ret in self.comprobante.retentions])
